@@ -24,6 +24,7 @@ import os
 import io
 from urllib.parse import urlparse
 import mimetypes
+from collections import defaultdict
 
 from nbxmpp.errors import StanzaError
 from nbxmpp.errors import MalformedStanzaError
@@ -68,6 +69,9 @@ class HTTPUpload(BaseModule):
         self._session.props.ssl_strict = False
         self._session.props.user_agent = f'Gajim {app.version}'
 
+        self._running_transfers: dict[
+            tuple[str, JID], set[HTTPFileTransfer]] = defaultdict(set)
+
     def _set_proxy_if_available(self) -> None:
         proxy = get_account_proxy(self._account)
         if proxy is None:
@@ -95,8 +99,11 @@ class HTTPUpload(BaseModule):
                                          GLib.FormatSizeFlags.IEC_UNITS)
             self._log.info('Component has a maximum file size of: %s', size)
 
-        for ctrl in app.window.get_controls(account=self._account):
-            ctrl.update_actions()
+    def get_running_transfers(self,
+                              contact: types.ChatContactT
+                              ) -> Optional[set[HTTPFileTransfer]]:
+
+        return self._running_transfers.get((contact.account, contact.jid))
 
     def make_transfer(self,
                       path: str,
@@ -133,17 +140,26 @@ class HTTPUpload(BaseModule):
         mime = mimetypes.MimeTypes().guess_type(path)[0]
         if not mime:
             mime = 'application/octet-stream'  # fallback mime type
-        self._log.info("Detected MIME type of file: %s", mime)
+        self._log.info('Detected MIME type of file: %s', mime)
 
-        return HTTPFileTransfer(self._account,
-                                path,
-                                contact,
-                                mime,
-                                encryption,
-                                groupchat)
+        transfer = HTTPFileTransfer(self._account,
+                                    path,
+                                    contact,
+                                    mime,
+                                    encryption,
+                                    groupchat)
+
+        key = (contact.account, contact.jid)
+        self._running_transfers[key].add(transfer)
+
+        return transfer
 
     def cancel_transfer(self, transfer: HTTPFileTransfer) -> None:
         transfer.set_cancelled()
+
+        key = (transfer.account, transfer.contact.jid)
+        self._running_transfers[key].discard(transfer)
+
         message = self._queued_messages.get(id(transfer))
         if message is None:
             return
@@ -264,8 +280,12 @@ class HTTPUpload(BaseModule):
                    message: Soup.Message,
                    transfer: HTTPFileTransfer
                    ) -> None:
+
         self._queued_messages.pop(id(transfer), None)
 
+        key = (transfer.account, transfer.contact.jid)
+        self._running_transfers[key].discard(transfer)
+
         if message.props.status_code == Soup.Status.CANCELLED:
             self._log.info('Upload cancelled')
             return
