@@ -23,13 +23,14 @@ from gi.repository import Gdk
 
 from nbxmpp.errors import StanzaError
 from nbxmpp.task import Task
+from nbxmpp.protocol import JID
 
 from gajim.common import app
 from gajim.common import ged
 from gajim.common.const import MUC_CREATION_EXAMPLES
 from gajim.common.const import MUC_DISCO_ERRORS
 from gajim.common.events import AccountConnected
-from gajim.common.events import AccountDisonnected
+from gajim.common.events import AccountDisconnected
 from gajim.common.ged import EventHelper
 from gajim.common.helpers import get_random_muc_localpart
 from gajim.common.helpers import validate_jid
@@ -85,7 +86,7 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
         self.set_focus(self._ui.address_entry)
 
     def _on_account_state(self,
-                          _event: Union[AccountConnected, AccountDisonnected]
+                          _event: Union[AccountConnected, AccountDisconnected]
                           ) -> None:
         any_account_connected = app.get_number_of_connected_accounts() > 0
         if any_account_connected:
@@ -131,7 +132,7 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
         self._ui.name_entry.set_placeholder_text(
             _('e.g. %s') % placeholder[0])
         self._ui.description_entry.set_placeholder_text(
-            _('e.g. %s' % placeholder[1]))
+            _('e.g. %s') % placeholder[1])
         self._ui.address_entry.set_placeholder_text(
             f'{placeholder[2]}@{server}')
 
@@ -208,6 +209,7 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
             return
 
         name = entry.get_text()
+        name = name.replace(' ', '-')
         server = self._get_muc_service_jid()
         self._ui.address_entry.set_text(f'{name.lower()}@{server}')
 
@@ -230,6 +232,10 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
 
     def _on_public_private_toggled(self, _radiobutton: Gtk.RadioButton) -> None:
         is_public = self._ui.public_radio.get_active()
+        if is_public:
+            self._validate_jid(self._ui.address_entry.get_text())
+        else:
+            self._ui.create_button.set_sensitive(True)
         self._ui.address_label.set_visible(is_public)
         self._ui.address_entry.set_visible(is_public)
 
@@ -289,10 +295,15 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
 
         # Create new group chat by joining
         assert self._account is not None
-        app.interface.create_groupchat(
-            self._account,
-            str(room_jid),
-            config)
+
+        if app.window.chat_exists(self._account, JID.from_string(room_jid)):
+            log.error('Trying to create groupchat '
+                      'which is already added as chat')
+            self.destroy()
+            return
+
+        client = app.get_client(self._account)
+        client.get_module('MUC').create(room_jid, config)
 
         self.destroy()
 
